resolution_width = 1280--default window size
resolution_height = 720
camera_x = 6000
camera_y = 3000
camera_z = 0.09--zoom
camera_r = 0--currently unused
gp_alphasorting = 0

local canvas_formats = love.graphics.getCanvasFormats()

gp_position_format = "rgba32f"--16 is much faster but has severe accuracy issues currently
gp_velocity_format = "rgba16f"--32 provides more variety, but usually isn't worth the wasted vram
--in order to use rgba16f for position formatting (i.e mobile devices), the final canvas resolution needs to be 128x128 or lower to look authentic
--stretch the canvas over the screen, rather than using full screen resolution like we are here

local particle_pixelscale = 25--25 pixels to 1, needs to match the particle systems scale or conversion from screen space to particle space will be incorrect

time = love.timer.getTime()*1000 --time in milliseconds

require "particle"

color_none = {0,0,0,0}
color_black = {0,0,0,255}
color_white = {255,255,255,255}

debug_particles = 0

--physics shaders
gravity_on = love.graphics.newShader("particle.glsl")
gravity_off = love.graphics.newShader("particle_off.glsl")
shader_subtract = love.graphics.newShader([[

vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords )
{
	return vec4(0,0,0,0);
}

]])

gp_gravity = false--gravity is controlled via the shader rather than supplying it via extern

wmb = 0--mouse wheel

function love.load() 
	desktop_width = love.graphics.getWidth()
	desktop_height = love.graphics.getHeight()
	print("Detected desktop resolution: " .. desktop_width .. "," .. desktop_height )
	
	local limits = love.graphics.getSystemLimits()
	print("Render-to-texture limit: " .. limits.canvasmsaa)
    print("Texture size limit: " .. limits.texturesize)
	print("Texture formats supported:")
	for k,v in pairs(canvas_formats) do
		print("   " .. k)
	end

	if not canvas_formats["rgba32f"] and canvas_formats["rgba16f"] then
		print("GPu warning: RGBA 32f unsupported, falling back to RGBA 16f!")
		gp_position_format = "rgba16f"
	elseif not canvas_formats["rgba32f"] then
		error("GPu: required RGBA 32f texture mode is unsupported by your video card.")
	end
	
	if resolution_width == 0 or resolution_height == 0 then
		resolution_width = desktop_width
		resolution_height = desktop_height
	end
	
	if(desktop_width == resolution_width and desktop_height == resolution_height) or (resolution_width == 0 and resolution_height == 0) then
		
	else

		love.window.setMode(resolution_width, resolution_height, {vsync=false,resizable=true, borderless=false, msaa=0, fullscreen=false})
		
		print("Changing resolution: " .. love.graphics.getWidth() .. ","  .. love.graphics.getHeight())
	end
	
	Init()
end

function love.update(dt)
	del = dt
	time = love.timer.getTime()*1000

	if love.keyboard.isDown("w") then
		camera_y = camera_y - 10000*del
	end
	if love.keyboard.isDown("s") then
		camera_y = camera_y + 10000*del
	end
	if love.keyboard.isDown("a") then
		camera_x = camera_x - 10000*del
	end
	if love.keyboard.isDown("d") then
		camera_x = camera_x + 10000*del
	end
	if love.keyboard.isDown("q") then
		camera_r = camera_r - 1*del
	end
	if love.keyboard.isDown("e") then
		camera_r = camera_r + 1*del
	end
	
	if wmb > 0 then
		camera_z = camera_z + 0.005
		wmb = wmb - 1
	elseif wmb < 0 then
		camera_z = camera_z - 0.005
		wmb = wmb + 1
		if camera_z < 0.005 then
			camera_z = 0.005
		end
	end
	
	local spawn = 0
	if love.mouse.isDown(1) then
		spawn = 1
	elseif love.mouse.isDown(2) then
		spawn = 2
	else
		spawn = 0
	end
	
	--change which particle is spawning this frame: uploading a table and distributing particle types based on need would be superior for a large particle system
	for k,g in pairs(gp) do
		if g.particleCountSQRT > 64 then
			g.physicsShader:send("spawn",spawn)
		elseif spawn == 2 then
			g.physicsShader:send("spawn",spawn)
		else
			g.physicsShader:send("spawn",0.0)
		end
	end
	
end

function love.wheelmoved( x, y )
	if y > 0 then--mouse wheel in
		if wmb < 1 then
			wmb = 1
		else
			wmb = wmb + 1
		end
	elseif y < 0 then--mouse wheel out
		if wmb > 1 then
			wmb = -1
		else
			wmb = wmb - 1
		end
	end
end

function love.keypressed(key)
	if key == "escape" then
		love.event.push("quit")
	end
	if key == "f1" then
		debug_particles = debug_particles + 1
		if debug_particles > 3 then
			debug_particles = 0
		end
	end
	
	if key == "f2" then--toggle alphasorting
		if gp_alphasorting == 1 and gp[1].alphaSort then
			gp_alphasorting = 0
			for k,g in pairs(gp) do
				g.alphaSort = false
				g.blend_mode = "add"
				g.particle_brush = love.graphics.newImage("particle.png")
			end
		elseif gp_alphasorting == 0 and gp[1].alphaSort == false then
			gp_alphasorting = 1
			for k,g in pairs(gp) do
				g.alphaSort = true
				g.blend_mode = "alpha"
				g.particle_brush = love.graphics.newImage("particle_opaque.png")
			end
		end
	
	end
	
	if key == "f3" then--swap shader in realtime
	
		if gp_gravity == true then
			gp_gravity = false
			
			particle_age, particle_uv = particle_data(1)
			
			gravity_off:send("particle_age",unpack(particle_age))
			
			--we modify all large particle system shaders
			for k,g in pairs(gp) do
				if g.particleCountSQRT > 64 then
					g.physicsShader = gravity_off

					g.mainShader:send("particle_uv",unpack(particle_uv))
				end
			end
			
		else
			gp_gravity = true
			particle_age, particle_uv = particle_data(1)
			
			gravity_on:send("particle_age",unpack(particle_age))
			
			for k,g in pairs(gp) do
				if g.particleCountSQRT > 64 then
					g.physicsShader = gravity_on

					g.mainShader:send("particle_uv",unpack(particle_uv))
				end
			end
			
		end
		
	end
end

function love.draw()
	--draw and scale our scene to the collision texture
	love.graphics.setCanvas(collision_texture)
	love.graphics.clear()
	love.graphics.push()

	love.graphics.translate(width/2,height/2)
	love.graphics.scale(camera_z)
	love.graphics.translate(-width/2,-height/2)
	
	love.graphics.setColor(127,127,127,255)
	love.graphics.rectangle("fill",1500-camera_x,1500-camera_y,3000,3000)
	love.graphics.setColor(255,255,255,255)
	love.graphics.setShader(shader_subtract)
	love.graphics.setBlendMode("replace","premultiplied")
	love.graphics.rectangle("fill",1800-camera_x,1800-camera_y,2400,3200)
	love.graphics.setShader()
	love.graphics.setBlendMode("alpha")
	love.graphics.setColor(127,127,127,255)
	love.graphics.circle("fill",8000-camera_x,3500-camera_y,1500)
	love.graphics.draw(terrain,(width/2*10)-camera_x,(height/2+height/6)*10-camera_y,0,60,20,128,128)
	love.graphics.setCanvas()
	love.graphics.pop()
	
	love.graphics.setColor(255,255,255,255)
	love.graphics.setBlendMode("alpha")
	love.graphics.clear(5,5,5,255)--background color
	
	love.graphics.setCanvas(mainCanvas)--particle final canvas
	love.graphics.clear(0,0,0,255)--canvas must be black to use additive blending

	local particles_rendered = 0
	local collision = true
	local cx,cy,cz = camera_x,camera_y,camera_z
	
	local mx, my = love.mouse.getX(), love.mouse.getY()
	local p_count = 0
	
		for k,g in pairs(gp) do
			if collision == true then
			
				-- create the collision texture once from an rgba image
				particle_generate_collision(g,collision_texture,cx,cy,cz)
				
				local col_tex = g.cnvs_col
				
				for k2,v2 in pairs(gp) do
					if v2 ~= g then
						if v2.cnvs_col == g.cnvs_col then
							particle_update(v2,cx,cy,cz)--update collision texture position for other particle systems using the same collision texture
						end
					end
				end
				collision = false-- avoid unnecessarily recalculating collision generation
			end
			
			--add some spawn randomization because our gpu-side randomization is quite predictable
			local x_mod = p_count * (width/20) + math.random(10)-5
			local y_mod = math.random(10)-5
			
			--translate screen coordinates + mouse to particle coordinates: mx/my are mouse coordinates here
			local x = (x_mod+mx+(g.particle_pixelscale*camera_z))/g.width
			local y = (my+(g.particle_pixelscale*camera_z))/g.height
			
			x = x - 0.5
			y = y - 0.5
			x = x / (g.particle_pixelscale*camera_z)
			y = y / (g.particle_pixelscale*camera_z)
			x = x + 0.5
			y = y + 0.5
		
			if g.particleCount > 32000 then--lazy way to determine which particle system is which, so that we can spawn more particles per frame on the larger system
				g.physicsShader:send("spawnCount", math.floor(100000*del))
			else
				g.physicsShader:send("spawnCount", math.ceil(5000*del))
			end
			
			g.physicsShader:send("mousePos", {x,y})--relative screen coordinates -0.5 to 1.5
			g.physicsShader:send("dt",del*100)
			p_count = p_count + 1

			particles_rendered = particles_rendered + g.particleCount
			particle_render(g,cx,cy,cz)
		end
		
	love.graphics.setCanvas()
	love.graphics.setColor(255,255,255,255)
	love.graphics.setBlendMode("alpha")
	
	--computed collision texture:
	if debug_particles == 2 or debug_particles == 3 then
		love.graphics.setShader(gp[1].distRender3)
		love.graphics.setColor(255,255,255,255)
		love.graphics.draw(gp[1].cnvs_col)
		love.graphics.setShader()
		
		love.graphics.setBlendMode("add")
		love.graphics.draw(mainCanvas)
		love.graphics.setBlendMode("alpha")
	else
		love.graphics.draw(mainCanvas)
	end
	love.graphics.setBlendMode("alpha")
	
	--draw original collision texture
	love.graphics.draw(collision_texture)
	
	if debug_particles == 1 or debug_particles == 3 then
		love.graphics.setShader()
		love.graphics.setBlendMode("alpha")
		
		love.graphics.push()
		local pos = 0
		local largest_y = 0
		
		for _,g in pairs(gp) do
		
			if g.particleCountSQRT*2 > largest_y then
				largest_y = g.particleCountSQRT*2
			end
			
			love.graphics.print(_,pos+5)
			love.graphics.draw(g.cnvs_posA, pos, 0, 0, 1, 1)
			-- love.graphics.draw(g.cnvs_velA, pos, 0, 0, 1, 1)

			love.graphics.line(pos,0,pos,g.particleCountSQRT)
			
			pos = pos + g.particleCountSQRT
			
			if pos > width then
				pos = 0
				love.graphics.translate(0,largest_y)
				largest_y = 0
			end
		end
		love.graphics.pop()
	end
	
	if debug_particles ~= 1 and debug_particles ~= 3 then
		love.graphics.print("FPS:"..love.timer.getFPS(),5,5)
		local vram = love.graphics.getStats()
		
		love.graphics.print("VRAM used:"..math.floor(vram.texturememory/100000)/10 .. ' MB',75,5)
		love.graphics.print("X:"..camera_x,5,25)
		love.graphics.print("Y:"..camera_y,5,45)
		love.graphics.print("Z:"..camera_z,5,65)
		love.graphics.print("debug_particles ( F1 to toggle ): " .. tostring(debug_particles), 5, 85)
		love.graphics.print("alpha sorting ( F2 to toggle ): " .. tostring(gp_alphasorting), 5, 105)
		love.graphics.print("gravity ( F3 to toggle ): " .. tostring(gp_gravity), 5, 125)
		love.graphics.print("total particle count:" .. particles_rendered, 5, 145)
		love.graphics.print("WSAD ( move camera )" , 5, 165)
		love.graphics.print("mouse wheel ( zoom )" , 5, 185)
		
		if gp_position_format == "rgba16f" then
			love.graphics.setColor(255,127,127,255)
			love.graphics.print("canvas format: " .. gp_position_format .. " (warning: low precision)",5,205)
			love.graphics.setColor(255,255,255,255)
		else
			love.graphics.print("canvas format: " .. gp_position_format,5,205)
		end
	
	else
		
		for a = 1, #gp do
			local x_mod = (a-1) * (width/20) 
			love.graphics.print(a, love.mouse.getX() + x_mod, love.mouse.getY()+16)
		end
		
		love.graphics.print("debug_particles ( F1 to toggle ): " .. tostring(debug_particles), 5, height-20)
	end
end

function love.resize(w, h)
	print("Window resized to " .. w .. "x" .. h ..".")
	local old_width = width
	local old_height = height

	width = w
	height = h
	
	resolution_width = w
	resolution_height = h
	
	canvas_resize()--reconstruct particle canvases if required
end

function Init()
	terrain = love.graphics.newImage("terrain.png")
	width = love.graphics.getWidth()
	height = love.graphics.getHeight()

	canvas_resize()
	
end

function create_particle_systems()
		print("Constructing GP particle tables and canvases using resolution: " .. width .. " / " .. height)
		
		--collision
		colCanvas = love.graphics.newCanvas(width,height,"rgba16f")
		
		--generic buffer canvas to flip fbos
		bufferCanvas = love.graphics.newCanvas(width,height,"rgba16f")
		
		gp_gravity = false
		
		local p_phys = love.graphics.newShader("particle_off.glsl")--no gravity shader
		local p_frag = love.graphics.newShader("particle_frag.glsl","particle_vert.glsl")
		
		local p_phys2 = love.graphics.newShader("particle2.glsl")--smaller particle system
		local p_frag2 = love.graphics.newShader("particle_frag.glsl","particle_vert2.glsl")
		
		local particle_age = {}
		local particle_age_2 = {}
		local particle_uv = {}
		local particle_uv_2 = {}
		
		particle_age, particle_uv = particle_data(1)
		particle_age_2, particle_uv_2 = particle_data(2)
	
		--distance field texture
		local buffer16f = love.graphics.newCanvas(1024,1024,"rgba16f")
		
		--[[construct particle systems]]
		gp = {}
		-- gp[1] = particle_new(p_phys, p_frag,buffer16f,bufferCanvas,mainCanvas,colCanvas,particle_age, particle_uv,"add", 1024)--1 million particles: if alpha sorting is required, avoid using it with a large system
		
		gp[1] = particle_new(p_phys, p_frag,buffer16f,bufferCanvas,mainCanvas,colCanvas,particle_age, particle_uv,"add", 256)	--65k particle system
		
		gp[2] = particle_new(p_phys2, p_frag2,buffer16f,bufferCanvas,mainCanvas,colCanvas,particle_age_2, particle_uv_2,"add", 64) --4k particle system
		for a = 3,8 do
			gp[a] = particle_new(p_phys2, p_frag2,buffer16f,bufferCanvas,mainCanvas,colCanvas,particle_age_2, particle_uv_2,"add", 64, gp[2].particle_mesh)--uses same mesh, since we are using the same amount of particles
		end
end

function particle_data(ty)--must be uploaded to each shader to describe how quickly the particle ages and what texture it's using
	local age = {}
	local uv = {}
	
	for a = 1, 20 do--the large shader supports up to 20 particle descriptions, although only 2 are currently used
		age[a] = 0.0
		uv[a] = {0.0,0.0}
	end
		
	if ty == 1 then
		age[1] = 0.0001--particle ages slowly
		uv[1] = {0.5,0.0}--use the right most texture from particle.png: a triangle
		age[2] = 0.01
		uv[2] = {0.0,0.0}--use left most texture: a circle
	else
		age[1] = 0.01
		uv[1] = {0.0,0.0}
		age[2] = 0.01
		uv[2] = {0.0,0.0}
	end
	
	return age,uv
end

function canvas_resize()

	mainCanvas = love.graphics.newCanvas(width,height,"normal",0)
	collision_texture = love.graphics.newCanvas(width,height)
	
	--create global particle table
	if not gp then
	
		create_particle_systems()

	else
		--reconstruct particle canvases
		local old_colCanvas, old_bufferCanvas = nil,nil
	
		if colCanvas then
			old_colCanvas = colCanvas
		end
		if bufferCanvas then
			old_bufferCanvas = bufferCanvas
		end

		colCanvas = love.graphics.newCanvas(width,height,"rgba16f")
		bufferCanvas = love.graphics.newCanvas(width,height,"rgba16f")

		for k,v in pairs(gp) do
			v.width = width
			v.height = height
			if v.cnvs_col == old_colCanvas then
				-- print("replacing colCanvas")
				v.cnvs_col = colCanvas
			end
			v.bufferCanvas = bufferCanvas
			v.particleCanvas = mainCanvas
		end
		
		old_colCanvas = nil
		old_bufferCanvas = nil
	end
	
end
