
/*
MIT License
Copyright (c) 2016 Jonathan Brodsky
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

Modified
*/

extern vec2 mousePos;
extern float spawnOffset;
extern Image velTexture;
extern Image colTexture;
extern vec2 translation;
//extern Image curlNoise;
extern float particleCount;
extern float particleCountSQRT;
extern float zscale;
extern float spawnCount;
extern float dt;

const float PI = 3.141592;
const float PI2 = 3.141592*2;

extern float particle_age[20];

extern float spawn;

float rand(vec2 co){
		return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}
//normalize
float atan2(vec2 v)
{
	return v.x == 0.0 ? sign(-v.y)*PI/2 : atan(-v.y, v.x);
}
float atan2(float x, float y)
{
    return x == 0.0 ? sign(-y)*PI/2 : atan(-y, x);
}

vec2 remap(vec2 floatdata) {//16 bit texture only
     return floatdata * (65536.0-1.0) / 65536.0 * 2.0 - 1.0;
}

/*
distance field calculation
A particle checks for a collision simply by sampling the obstacle texture. If it finds a normal at its location, it changes its velocity using the shader function reflect. 

Sometimes particles end up on/in an obstacle with a low or zero velocity. To dislodge these they should be given a little nudge in the direction of the normal, pushing them away from the obstacle. 

*/
void effects(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords)
{
  float IspawnOffset = mod(spawnOffset*spawnCount, particleCount);
  float particleIndex = mod(texture_coords.x*particleCountSQRT+texture_coords.y*particleCount, particleCount);
  bool spawning = particleIndex>IspawnOffset && particleIndex < IspawnOffset+spawnCount && spawn > 0.0;
  vec4 o = Texel(texture, texture_coords);
  vec3 position = o.xyz;
  float age = o.a;
  
  int type = int(o.b*0.5);

  if(!spawning && age < 0.001)
    discard;

  vec4 vTex = Texel(velTexture, texture_coords);
  
  position.xy += translation;
  
  age -= particle_age[type]*dt;

  
  //recalculated from vertex shader
  float size = (sin(o.a*3.14) * 15)*0.5;
  

  vec2 render_pos = position.xy+(vTex.xy*size);
  
  vec4 cTex = Texel(colTexture, 0.5+(render_pos-0.5)*zscale);
  
  // float cDist = 1.0-length(cTex.rgb);
  vec2 distvec = remap(cTex.rg);
  float cDist = length(distvec)*130.0;
  
   if(cDist < 0.5)//0.25)
   {
		vec2 n = vec2(normalize(distvec));

		vTex.xy = (-2*dot(vTex.xy, n)*n+vTex.xy)*0.5;
   }

  position.xy += vTex.xy*dt;
  
  
  // slow down the particles
  //vTex.xy *= 0.94;
  // add gravity
  vTex.y += sin(spawnOffset*0.0001)*0.0001;

	if (type == 1)//upward fire	
	{
		vTex.y -= 0.0002*dt;
		//vTex.x *= 0.98;
		vTex.x = vTex.x - ((vTex.x / 1000)*dt);
	}
	else//type 0: directional vectors
	{
	
		//follow the mouse
	    float dist = length(mousePos-o.xy)*500;
	   
	    if(dist <= 0.0)
	    {
		   dist = 1;
	    }
	   
	    vTex.xy += ((mousePos-o.xy)*0.00004)*dt;
	}

  love_Canvases[0] = vec4(position, age);
  
  vTex.b = atan2(normalize(vTex.xy))/PI2;//rotation from velocity
  
  love_Canvases[1] = vTex;

  if(spawning){
	float pCount = (particleIndex-IspawnOffset)/spawnCount;
    love_Canvases[0] = vec4(mousePos.x, mousePos.y, spawn, pCount); // 
	
	float rnd = rand(screen_coords);
	float vx = (-0.0035+rnd*0.01)*0.3;
	float vy = (-0.0035+rand(screen_coords*2)*0.01)*0.3;

	love_Canvases[1] = vec4(vx,vy,atan2(normalize(vec2(vx,vy)))/PI2,0);//rnd*2);
	}
	
}
