
local GP_COLLISION_REFRESHRATE = 50--ms between distance field collision recalculation

--translation effect can be applied with a shader pass on cvns_ping without affecting any other shader calculations if necessary by copying cvns_pong to a buffer, applying the shader, then copying it back to cvns_pong
local translation_effect = love.graphics.newShader [[
	extern vec2 v;
	
	vec4 effect ( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords ) {
		
		vec4 col = Texel(texture, texture_coords);
		
		col.xy += v;
		
		return col;
	}
	
	]]

function particle_new(particle_physics,particle_fragvert,buffer16f,buffer_canvas,draw_canvas,col_canvas, p_age, p_uv,blend_mode,p_size,p_mesh,alphasort)
	
	local g = {}
		
	if not p_size then
		g.particleCountSQRT = 256
	else
		g.particleCountSQRT = p_size
	end
	 
	g.alphaSort = alphasort or false
	
	g.particle_physics = particle_physics
	g.particle_fragvert = particle_fragvert

	g.particle_drawCount = 0
	g.particleCount = g.particleCountSQRT*g.particleCountSQRT

	if not buffer16f then
		g.buffer16f = love.graphics.newCanvas(1024,1024,"rgba16f")
		print("GPu: Initialized gp with new buffer16f")
	else
		g.buffer16f = buffer16f
	end
	
	if buffer_canvas == nil then
		g.bufferCanvas = love.graphics.newCanvas(width,height,"rgba16f")
		print("GPu: Initialized gp with new buffer.")
	else
		g.bufferCanvas = buffer_canvas
	end
	
	if draw_canvas == nil then
		g.particleCanvas = love.graphics.newCanvas(g.width,g.height)
		print("GPu: Initialized gp with new render texture.")
	else
		g.particleCanvas = draw_canvas
	end
	
	if col_canvas ~= nil then
		g.cnvs_col = col_canvas
		distfield_init(g)
		print("GPu: Initialized gp with collision texture.")
	else
		print("GPu: Initialized gp with no collision texture.")
	end

	if not blend_mode then
		g.blend_mode = "add"
	else
		g.blend_mode = blend_mode
	end

	 -- generate the mesh data that will draw the particles out of the fbo
	 if not p_mesh then
		print("GPu: Autocreating a mesh..")
		particle_create_mesh(g)
	 else
		--mesh must have the same particle count
		g.particle_mesh = p_mesh
	 end
	
	g.particle_age = p_age
	g.particle_uv = p_uv
	g.intensity = 64--brightness 1-255
	particle_init(g)
	return g
end

function particle_create_mesh(g)
	-- generate the mesh data that will draw the particles out of the fbo
	local meshData = {}
	local vertMap = {}
	local numVerts = g.particleCount
	for i=0,(numVerts*4)-1 do
		table.insert(meshData,{0,0, 0,0, i%4, math.floor(i/4)})
	end
	for i=0,numVerts-1 do
		table.insert(vertMap, i*4+2)
		table.insert(vertMap, i*4+2)
		table.insert(vertMap, i*4+1)
		table.insert(vertMap, i*4+3)
		table.insert(vertMap, i*4+4)
		table.insert(vertMap, i*4+4)
	end
	
	g.particle_mesh = love.graphics.newMesh({{"VertexPosition", "float", 2}, {"VertexTexCoord", "float", 2}, {"PointCorner", "float", 1}, {"SpriteCount", "float", 1}}, meshData, "strip")
	g.particle_mesh:setVertexMap(vertMap)
end

function particle_init(g)--re-initialize particle system
	print("GPu: Initializing gp: " .. tostring(g))
	g.particle_sortCount = 211
	g.particle_pixelscale = 25/2
	g.distfield_time = 0
	g.distfield_colx = 0
	g.distfield_coly = 0
	g.distfield_colz = 1
	
	g.particle_brush = love.graphics.newImage("particle.png")
	g.physicsShader = g.particle_physics
	g.mainShader = g.particle_fragvert
	g.sortShader = love.graphics.newShader("oddeven_mergesort_frag.glsl")
	
	g.particle_pixelscale_vel = 100*(g.particle_pixelscale/25)
	
	g.physicsShader:send("particle_age",unpack(g.particle_age))
	g.mainShader:send("particle_uv",unpack(g.particle_uv))
	
	g.particle_last_x = 0
	g.particle_last_y = 0

	local function createCanvas(x,y,scale,type)
		local c = love.graphics.newCanvas(x or g.particleCountSQRT,y or g.particleCountSQRT,type or "rgba16f")
		c:setFilter(scale or "nearest", scale or "nearest")
		return c
	end
	
	-- generate the fbos
	g.cnvs_posA = createCanvas(nil,nil,nil,gp_position_format or "rgba32f")
	g.cnvs_posB = createCanvas(nil,nil,nil,gp_position_format or "rgba32f")
	g.cnvs_velA = createCanvas(nil,nil,nil,gp_velocity_format or "rgba16f")
	g.cnvs_velB = createCanvas(nil,nil,nil,gp_velocity_format or "rgba16f")

  end
  
 function particle_update(g,camera_x, camera_y, camera_z)--move the distance field texture
	g.distfield_colx = camera_x
	g.distfield_coly = camera_y
	g.distfield_colz = camera_z
 end
 
 function particle_generate_collision(g,tex,camera_x,camera_y,camera_z)--regenerate the distance field
		
	love.graphics.setBlendMode("replace","premultiplied")
	love.graphics.setShader()
	love.graphics.setStencilTest()
	love.graphics.setColor(255,255,255,255)

	if g.distfield_time < time then--recreate distance field

		love.graphics.setCanvas(g.cnvs_col)
		love.graphics.clear(color_none)
		g.distfield_time = time+GP_COLLISION_REFRESHRATE
		g.distfield_colx = camera_x
		g.distfield_coly = camera_y
		g.distfield_colz = camera_z

		tex:setWrap("repeat","repeat")
		
		distfield_render(g,tex,g.cnvs_col)

	else
		--render the old distance field texture
		love.graphics.setCanvas(g.bufferCanvas)
		love.graphics.clear(color_none)
		love.graphics.draw(g.cnvs_col)
		
		love.graphics.setCanvas(g.cnvs_col)
		love.graphics.clear(color_none)
				
		local sc = 1 - (g.distfield_colz - camera_z)*10
		
		local tx,ty = (g.distfield_colx - camera_x)*camera_z, (g.distfield_coly - camera_y)*camera_z
		
		tx = tx + g.width/2
		ty = ty + g.height/2
		
		tx = tx - sc*(g.width/2)
		ty = ty - sc*(g.height/2)

		love.graphics.draw(g.bufferCanvas, tx, ty,0,sc,sc)

		g.distfield_colx = camera_x
		g.distfield_coly = camera_y
		g.distfield_colz = camera_z
	end
		
 end
 
function particle_render(g,camera_x,camera_y,camera_z)

	-- for k,v in pairs(g) do
		-- print(tostring(k) .. ":" .. tostring(v))
	-- end

  -- setup the physics drawing pass
  -- need to use ping pong fbos, because fbos cant draw into themselves
	local cnvs_ping, cnvs_pong, cnvs_ping_v, cnvs_pong_v = g.cnvs_posA, g.cnvs_posB, g.cnvs_velA, g.cnvs_velB
	
	if g.particle_drawCount%2 == 1 then
		cnvs_ping, cnvs_pong, cnvs_ping_v, cnvs_pong_v = g.cnvs_posB, g.cnvs_posA, g.cnvs_velB, g.cnvs_velA
	end

	local vx,vy = 0,0

	vx,vy = g.particle_last_x - camera_x,g.particle_last_y - camera_y

	vx = ((vx)/((g.width/2)*g.particle_pixelscale_vel))*2
	vy = ((vy)/((g.height/2)*g.particle_pixelscale_vel))*2

	particle_last_x = camera_x
	particle_last_y = camera_y

	love.graphics.setBlendMode("replace","premultiplied")
	love.graphics.setShader()
	love.graphics.setStencilTest()
	love.graphics.setColor(255,255,255,255)

	--update particle translations
	g.physicsShader:send("translation",{vx,vy})
	g.mainShader:send("translation",{vx,vy})

	love.graphics.setCanvas()

	love.graphics.setBlendMode("replace", "premultiplied")
	love.graphics.setCanvas(cnvs_ping, cnvs_ping_v)
	love.graphics.setShader(g.physicsShader)
	g.physicsShader:send("spawnOffset", g.particle_drawCount)

	--translate mouse coordinates to particle coordinates
	-- local x = (love.mouse.getX()+(g.particle_pixelscale*camera_z))/g.width
	-- local y = (love.mouse.getY()+(g.particle_pixelscale*camera_z))/g.height
	-- x = x - 0.5
	-- y = y - 0.5
	-- x = x / (g.particle_pixelscale*camera_z)
	-- y = y / (g.particle_pixelscale*camera_z)
	-- x = x + 0.5
	-- y = y + 0.5
	-- g.physicsShader:send("mousePos", {x,y})
	
	g.physicsShader:send("colTexture", g.cnvs_col)
	g.physicsShader:send("zscale", g.particle_pixelscale*camera_z)
	g.physicsShader:send("velTexture", cnvs_pong_v)
	g.physicsShader:send("particleCount", g.particleCount)
	g.physicsShader:send("particleCountSQRT", g.particleCountSQRT)

	g.particle_last_x = camera_x
	g.particle_last_y = camera_y
	
	love.graphics.draw(cnvs_pong,0,0,0,1,1)

	g.particle_drawCount = g.particle_drawCount+1

  -- sorting pass on the particle positions
  -- reset the stage and pass values
  -- run through the sorting a few times
  
	if g.alphaSort == true then
		love.graphics.setBlendMode("replace", "premultiplied")
		love.graphics.setShader(g.sortShader)
		
		local stage = -1
		local pass = -1
		
		for i=1,20 do
			g.particle_sortCount = g.particle_sortCount+1
			if g.particle_sortCount > 150 then
			  stage = -1
			  pass = -1
			 g.particle_sortCount = 0;
			end
			
			love.graphics.setCanvas(cnvs_pong, cnvs_pong_v)
			
			pass = pass-1
			
			if pass<0 then
				stage = stage + 1
				pass = stage
			end
			local pstage = math.pow(2, stage);
			local ppass  = math.pow(2, pass);

			-- from the gpu gems code
			-- glUniform3fARB(oddevenMergeSort.getUniformLocation("Param1"), float(pstage+pstage), 
			--    float(ppass%pstage), float((pstage+pstage)-(ppass%pstage)-1));
			-- glUniform3fARB(oddevenMergeSort.getUniformLocation("Param2"), float(width), float(height), float(ppass));
			g.sortShader:send("Param1", {pstage+pstage, ppass%pstage, (pstage+pstage)-(ppass%pstage)-1})
			g.sortShader:send("Param2", {g.particleCountSQRT, g.particleCountSQRT, ppass})
			g.sortShader:send("velTexture", cnvs_ping_v)
			love.graphics.draw(cnvs_ping, 0,0,0,1,1)
			-- flip/flop canvases
			local tcnvs_ping = cnvs_pong
			cnvs_pong = cnvs_ping
			cnvs_ping = tcnvs_ping
			local tcnvs_ping_v = cnvs_pong_v
			cnvs_pong_v = cnvs_ping_v
			cnvs_ping_v = tcnvs_ping_v

		end
	end
	
	love.graphics.push()
	love.graphics.translate(g.width/2,g.height/2)
	love.graphics.scale(camera_z*g.particle_pixelscale)
	love.graphics.translate(-g.width/2,-g.height/2)
	
  -- draw the main mesh
	love.graphics.setCanvas(g.particleCanvas)


	love.graphics.setShader(g.mainShader)

	g.mainShader:send("posTexture", cnvs_ping)
	g.mainShader:send("velTexture", cnvs_ping_v)
	g.mainShader:send("particleCountSQRT", g.particleCountSQRT)

	g.particle_mesh:setTexture(g.particle_brush)
	
	if g.blend_mode == "add" then
		-- premultiplied
		love.graphics.setColor(255,255,255,g.intensity)
		love.graphics.setBlendMode("add","premultiplied")
	else
	
		--currently always rendering at maximum intensity because we aren't switching the shader when using alpha in the example
		love.graphics.setBlendMode(g.blend_mode)
	end
	love.graphics.draw(g.particle_mesh)
	love.graphics.pop()

	love.graphics.setBlendMode("alpha")
	love.graphics.setCanvas()
	love.graphics.setColor(255,255,255,255)
	love.graphics.setShader()
	
end

function particle_draw(g)--not used unless doing off-screen renders, usually render to screen directly using particle_render
	
	love.graphics.setBlendMode(g.blend_mode)--,"premultiplied")
	love.graphics.setColor(255,255,255,255)
	
	love.graphics.draw(g.particleCanvas)
	
	love.graphics.setBlendMode("alpha")
	love.graphics.setColor(255,255,255,255)
	
end


function distfield_init(g)
	g.distfield_time = time-1
	g.distfield_colx = 0
	g.distfield_coly = 0
	g.distfield_colz = 1
	g.distfield_texsize = 256
	g.distfield_steps = 7
	
	g.distShader3 = love.graphics.newShader([[
		// Jump flooding algorithm for EDT according
		// to Danielsson (1980) and Guodong Rong (2007).
		// Implementation by Stefan Gustavson 2010.
		// This code is in the public domain.

		// This code represents one iteration of the flood filling.
		// You need to run it multiple times with different step
		// lengths to perform a full distance transformation.

		uniform sampler2D texture;
		uniform float texw, texh;
		uniform float texlevels;
		uniform float stepu;
		uniform float stepv;

		// Helper functions to remap unsigned normalized floats [0.0,1.0]
		// coming from an integer texture to the range we need [-1, 1].
		// The transformations are very specifically designed to map
		// integer texel values exactly to pixel centers, and vice versa.
		// (See fragment_seed.glsl for details.)
		vec2 remap(vec2 floatdata) {
			 return floatdata * (texlevels - 1.0) / texlevels * 2.0 - 1.0;
		}

		vec2 remap_inv(vec2 floatvec) {
			 return (floatvec + 1.0)* 0.5 * texlevels / (texlevels - 1.0);
		}

		vec4 effect(vec4 fragColor, Image texture, vec2 uv, vec2 screen_coords)
		{
		  
		  // Search for better distance vectors among 8 candidates
		  vec2 stepvec; // Relative offset to candidate being tested
		  vec2 newvec;  // Absolute position of that candidate
		  vec3 newseed; // Closest point from that candidate (.xy) and its distance (.z)
		  vec3 bestseed; // Closest seed so far
		  bestseed.xy = remap(texture2D(texture, uv).rg);
		  bestseed.z = length(bestseed.xy);

		  // This code depends on the texture having a CLAMP_TO_BORDER
		  // attribute and a border color with R = 0.
		  // The commented-out lines handle clamping to the edge explicitly
		  // to avoid propagating incorrect vectors when looking outside
		  // of [0,1] in u and/or v.
		  // These explicit conditionals cause a slowdown of about 25%.
		  // Sometimes a periodic transform with edge repeats might be
		  // what you want. In that case, the texture wrap mode can be
		  // set to GL_REPEAT, and the shader code can be left unchanged.

		  stepvec = vec2(-stepu, -stepv);
		  newvec = uv + stepvec;
		//  if ( all( bvec4( lessThan(newvec, vec2(1.0)), greaterThan(newvec, vec2(0.0)) ) ) ) {
			newseed.xy = remap(texture2D(texture, newvec).rg);
			if(newseed.x > -0.99999) { // if the new seed is not "indeterminate distance"
			  newseed.xy = newseed.xy + stepvec;
			  newseed.z = length(newseed.xy);
			  if(newseed.z < bestseed.z) {
				bestseed = newseed;
			  }
			}
		//  }

		  stepvec = vec2(-stepu, 0.0);
		  newvec = uv + stepvec;
		//  if ( all( bvec4( lessThan(newvec, vec2(1.0)), greaterThan(newvec, vec2(0.0)) ) ) ) {
			newseed.xy = remap(texture2D(texture, newvec).rg);
			if(newseed.x > -0.99999) { // if the new seed is not "indeterminate distance"
			  newseed.xy = newseed.xy + stepvec;
			  newseed.z = length(newseed.xy);
			  if(newseed.z < bestseed.z) {
				bestseed = newseed;
			  }
			}
		//  }

		  stepvec = vec2(-stepu, stepv);
		  newvec = uv + stepvec;
		//  if ( all( bvec4( lessThan(newvec, vec2(1.0)), greaterThan(newvec, vec2(0.0)) ) ) ) {
			newseed.xy = remap(texture2D(texture, newvec).rg);
			if(newseed.x > -0.99999) { // if the new seed is not "indeterminate distance"
			  newseed.xy = newseed.xy + stepvec;
			  newseed.z = length(newseed.xy);
			  if(newseed.z < bestseed.z) {
				bestseed = newseed;
			  }
			}
		//  }

		  stepvec = vec2(0.0, -stepv);
		  newvec = uv + stepvec;
		//  if ( all( bvec4( lessThan(newvec, vec2(1.0)), greaterThan(newvec, vec2(0.0)) ) ) ) {
			newseed.xy = remap(texture2D(texture, newvec).rg);
			if(newseed.x > -0.99999) { // if the new seed is not "indeterminate distance"
			  newseed.xy = newseed.xy + stepvec;
			  newseed.z = length(newseed.xy);
			  if(newseed.z < bestseed.z) {
				bestseed = newseed;
			  }
			}
		//  }

		  stepvec = vec2(0.0, stepv);
		  newvec = uv + stepvec;
		//  if ( all( bvec4( lessThan(newvec, vec2(1.0)), greaterThan(newvec, vec2(0.0)) ) ) ) {
			newseed.xy = remap(texture2D(texture, newvec).rg);
			if(newseed.x > -0.99999) { // if the new seed is not "indeterminate distance"
			  newseed.xy = newseed.xy + stepvec;
			  newseed.z = length(newseed.xy);
			  if(newseed.z < bestseed.z) {
				bestseed = newseed;
			  }
			}
		//  }

		  stepvec = vec2(stepu, -stepv);
		  newvec = uv + stepvec;
		//  if ( all( bvec4( lessThan(newvec, vec2(1.0)), greaterThan(newvec, vec2(0.0)) ) ) ) {
			newseed.xy = remap(texture2D(texture, newvec).rg);
			if(newseed.x > -0.99999) { // if the new seed is not "indeterminate distance"
			  newseed.xy = newseed.xy + stepvec;
			  newseed.z = length(newseed.xy);
			  if(newseed.z < bestseed.z) {
				bestseed = newseed;
			  }
			}
		//  }

		  stepvec = vec2(stepu, 0.0);
		  newvec = uv + stepvec;
		//  if ( all( bvec4( lessThan(newvec, vec2(1.0)), greaterThan(newvec, vec2(0.0)) ) ) ) {
			newseed.xy = remap(texture2D(texture, newvec).rg);
			if(newseed.x > -0.99999) { // if the new seed is not "indeterminate distance"
			  newseed.xy = newseed.xy + stepvec;
			  newseed.z = length(newseed.xy);
			  if(newseed.z < bestseed.z) {
				bestseed = newseed;
			  }
			}
		//  }

		  stepvec = vec2(stepu, stepv);
		  newvec = uv + stepvec;
		//  if ( all( bvec4( lessThan(newvec, vec2(1.0)), greaterThan(newvec, vec2(0.0)) ) ) ) {
			newseed.xy = remap(texture2D(texture, newvec).rg);
			if(newseed.x > -0.99999) { // if the new seed is not "indeterminate distance"
			  newseed.xy = newseed.xy + stepvec;
			  newseed.z = length(newseed.xy);
			  if(newseed.z < bestseed.z) {
				bestseed = newseed;
			  }
			}
		//  }

		  return vec4(remap_inv(bestseed.xy), 0.0, 1.0);
		}

	]])
	
	g.seedShader = love.graphics.newShader([[
		// Jump flooding algorithm for EDT according
		// to Danielsson (1980) and Guodong Rong (2007).
		// Implementation by Stefan Gustavson 2010.
		// This code is in the public domain.

		// This shader initializes the distance field
		// in preparation for the flood filling.

		uniform sampler2D texture;
		uniform float texlevels;

		// Helper function to remap unsigned normalized floats [0.0..1.0]
		// coming from a texture stored in integer format internally to a
		// signed float vector pointing exactly to a pixel centre in texture
		// space. The range of valid vectors is
		// [-1.0+0.5/texsize, 1.0-0.5/texsize], with the special value
		// -1.0-0.5*texsize (represented as integer 0) meaning
		// "distance vector still undetermined".
		// The mapping is carefully designed to map both 8 bit and 16
		// bit integer texture data to distinct and exact floating point
		// texture coordinate offsets and vice versa.
		// 8 bit integer textures can be used to transform images up to
		// size 128x128 pixels, and 16 bit integer textures can be used to
		// transform images up to 32768x32768, i.e. beyond the largest
		// texture size available in current implementations of OpenGL.
		// Direct use of integers in the shader (by means of texture2DRect
		// and GL_RG8I and GL_RG16I texture formats) could be faster, but
		// this code is conveniently compatible even with version 1.2 of GLSL
		// (i.e. OpenGL 2.1), and the main shader is limited by texture access
		// and branching, not ALU capacity, so a few extra multiplications
		// for indexing and output storage are not that bad.
		
		vec2 remap(vec2 floatdata) {
			 return floatdata * (texlevels-1.0) / texlevels * 2.0 - 1.0;
		}

		vec4 effect(vec4 fragColor, Image texture, vec2 texture_coords, vec2 screen_coords)
		{
			vec2 uv = texture_coords;
			
		  float pixel = texture2D(texture, uv).a;
		  float myzero = 0.5 * texlevels / (texlevels - 1.0); // Represents zero
		  float myinfinity = 0.0;                             // Represents infinity
		  // Pixels > 0.5 are objects, others are background
		  return vec4(pixel < 0.5 ? myinfinity : myzero);
		}
	]])
	
	g.distRender3 = love.graphics.newShader([[
	
		uniform sampler2D texture;
		uniform float texlevels;

		vec2 remap(vec2 floatdata) {
			 return floatdata * (texlevels-1.0) / texlevels * 2.0 - 1.0;
		}

		vec4 effect(vec4 fragColor, Image texture, vec2 uv, vec2 screen_coords)
		{
		  vec2 distvec = remap(texture2D(texture, uv).rg);
		  vec2 rainbow = 0.5+0.5*(normalize(distvec));
		  
		  //return vec4(1.0-length(distvec)*50.0,1.0-length(distvec)*50.0, 1.0-length(distvec)*50.0, 1.0);
		  return vec4(rainbow, 1.0-length(distvec)*4.0, 1.0);
		}

	]])

	love.graphics.setColor(255,255,255,255)
	g.iChannel0 = love.graphics.newCanvas(g.distfield_texsize,g.distfield_texsize,"rgba16f")
	g.iChannel0:setFilter("nearest","nearest")
	g.iChannel0:setWrap("clampzero", "clampzero")

	g.seedShader:send("texlevels", 65536.0)--16f
	
	g.distShader3:send("texlevels", 65536.0)
	g.distRender3:send("texlevels", 65536.0)
	
	g.width = love.graphics.getWidth()
	g.height = love.graphics.getHeight()

end

function distfield_render(g, tex,cnvs)

	local bl = love.graphics.getBlendMode()
	
	love.graphics.setCanvas(g.iChannel0)
	love.graphics.clear(color_none)
	
	love.graphics.setCanvas(g.buffer16f)
	
	love.graphics.setBlendMode("replace","premultiplied")
	love.graphics.setColor(255,255,255,255)
	love.graphics.draw(g.iChannel0,0,0,0)
	
	love.graphics.setColor(255,255,255,255)
		
	love.graphics.setCanvas(g.iChannel0)
	love.graphics.clear(color_none)
	love.graphics.setShader(g.seedShader)

	local w,h = tex:getWidth()/2,tex:getHeight()/2
	-- w = w * (g.height/g.width)--maintain ratio: can cause distortion using non flat coordinates

	local sc_x, sc_y = g.distfield_texsize/g.width,g.distfield_texsize/g.height
	
	love.graphics.push()

	love.graphics.draw(tex,0,0,0,sc_x,sc_y,0,0)

	love.graphics.pop()
	love.graphics.setShader()

	local max_step = g.distfield_steps

	for a = 1, max_step do 

		g.distShader3:send("stepu", a / h)
		g.distShader3:send("stepv", a / w)

		love.graphics.setShader(g.distShader3)
		love.graphics.setCanvas(g.buffer16f)
		love.graphics.draw(g.iChannel0,0,0,0)
		
		love.graphics.setShader()
		
		love.graphics.setCanvas(g.iChannel0)
		love.graphics.draw(g.buffer16f,0,0,0)
	end

	love.graphics.setCanvas(cnvs)
	-- love.graphics.setShader(distRender3)
	love.graphics.setColor(255,255,255,255)
	love.graphics.draw(g.iChannel0,0,0,0,g.width/g.distfield_texsize,g.height/g.distfield_texsize)
	love.graphics.setShader()
	love.graphics.setBlendMode(bl)

end

